/* УСТРОЙСТВО ВЕСОИЗМЕРИТЕЛЬНОЕ ТИПА CAS CI
 * (c) Copyright 2020, GLONASSsoft
 * This file is provided as is (no warranties).
 */

#pragma library CasCI

#include serial
#include string

#define US_STATUS_BIT 0x00 /* Нестабильно */
#define ST_STATUS_BIT 0x01 /* Cтабильно */
#define OL_STATUS_BIT 0x02 /* Перегрузка */

#define GS_STATUS_BIT 0x00 /* Вес брутто */
#define NT_STATUS_BIT 0x04 /* Вес нетто */

/* Функция чтения периодических данных из порта */
stock LibraCiRead(SerialPort:port = SERIAL_PORT_0, Buff[], SizeBuff)
{
    new Len, Offs = 0;
    new BuffRead[22 * 3];  
    new pBuff = 0;
    
    Len = rsrecv(BuffRead, _, 100, 100, port); /* Читаем буфер */
    for (new i = 0; i < Len; i++)
    {
        if (BuffRead[i] == 0x0D)
        {
            Offs = (i + 1);
            break;
        }
    }
    
    while ((pBuff < SizeBuff) && (Offs < Len))
    {
        Buff[pBuff++] = BuffRead[Offs++];
        if (BuffRead[Offs] == 0x0D) break;
    }
    printf("Data len=%d", pBuff);
    return pBuff
}

/* Разобрать пакет типа 22 байта */
stock bool:LibraCiParse22Byte(DataBuff[], DataLen, &Weight, &WeightPow, &StatusBits)
{
  new Value = 0;
  new Pow = 0;
  new Bits = 0;
  new bool:Negative = false;
  new bool:Dot = false;

  if (DataLen != 21) return false;
  /* Значение веса */
  for(new i = 0; i < 8; i++)
  {
    if (DataBuff[9+i] == '-') Negative = true;
    if (DataBuff[9+i] == '.') Dot = true;
    if ('0' <= DataBuff[9+i] <= '9')
    {
      Value = (Value * 10);
      Value += (DataBuff[9+i] - '0');
      if (Dot) Pow++;
    }
  }
  if (Negative) Value = -Value;
  Weight = Value;
  WeightPow = Pow;
  /* Флаги с информацией о измерении */
  if (strfind(DataBuff, "US", false) != -1) Bits |= US_STATUS_BIT;
  else if (strfind(DataBuff, "ST", false) != -1) Bits |= ST_STATUS_BIT;
  else if (strfind(DataBuff, "OL", false) != -1) Bits |= OL_STATUS_BIT;

  if (strfind(DataBuff, "GS", false) != -1) Bits |= GS_STATUS_BIT;
  else if (strfind(DataBuff, "NT", false) != -1) Bits |= NT_STATUS_BIT;
  StatusBits = Bits;

  return true;
}

/* Разобрать пакет типа 10 байта */
stock bool:LibraCiParse10Byte(DataBuff[], DataLen, &Weight, &WeightPow)
{
  new Value = 0;
  new Pow = 0;
  new bool:Negative = false;
  new bool:Dot = false;

  if (DataLen != 9) return false;
  /* Значение веса */
  for(new i = 0; i < 8; i++)
  {
    if (DataBuff[i] == '-') Negative = true;
    if (DataBuff[i] == '.') Dot = true;
    if ('0' <= DataBuff[i] <= '9')
    {
      Value = (Value * 10);
      Value += (DataBuff[i] - '0');
      if (Dot) Pow++;
    }
  }
  if (Negative) Value = -Value;
  Weight = Value;
  WeightPow = Pow;

  return true;
}

/* Разобрать пакет типа 10 байта */
stock bool:LibraCiParse18Byte(DataBuff[], DataLen, &Weight, &WeightPow, &StatusBits)
{
  new Value = 0;
  new Pow = 0;
  new bool:Negative = false;
  new bool:Dot = false;
  new Bits = 0;

  if (DataLen != 17) return false;
  /* Значение веса */
  for(new i = 0; i < 8; i++)
  {
    if (DataBuff[6+i] == '-') Negative = true;
    if (DataBuff[6+i] == '.') Dot = true;
    if ('0' <= DataBuff[6+i] <= '9')
    {
      Value = (Value * 10);
      Value += (DataBuff[9+i] - '0');
      if (Dot) Pow++;
    }
  }
  if (Negative) Value = -Value;
  Weight = Value;
  WeightPow = Pow;
  /* Флаги с информацией о измерении */
  if (strfind(DataBuff, "US", false) != -1) Bits |= US_STATUS_BIT;
  else if (strfind(DataBuff, "ST", false) != -1) Bits |= ST_STATUS_BIT;
  else if (strfind(DataBuff, "OL", false) != -1) Bits |= OL_STATUS_BIT;

  if (strfind(DataBuff, "GS", false) != -1) Bits |= GS_STATUS_BIT;
  else if (strfind(DataBuff, "NT", false) != -1) Bits |= NT_STATUS_BIT;
  StatusBits = Bits;

  return true;

  return true;
}


stock LibraCiSendReq(SerialPort:port, Addr)
{
  new Buff[1]

  Buff[0] = DevId; /* Данные обмена в ждущем режиме поступают только после передачи на устройство его номера */
  /* Передаем запрос */
  rssend(Buff, 1, port)
}



