/* Libra NVT-9 functions
 * ver 1.0
 * (c) Copyright 2022 GLONASSsoft
 * This file is provided as is (no warranties).
 *
 * Библиотека предназначена для работы с весовым индикатором НВТ-9
 * Данные из индикатора передаются в непрерывном режиме в коде ASCII в COM-порт ПЭВМ 
 * Настройки порта 9600 8-N-1 
 */
#pragma library LibraNvt9

#include serial

#define NVT9_RECV_LEN 14
#define NVT9_RECV_TO  100

/* Функция чтения данных из порта */
stock LibraNvt9Read(SerialPort: Port = SERIAL_PORT_0, Buff[], Size = sizeof(Buff))
{
  new Ch[1]
  new Len
  
  Len = 0
  while (rsrecv(Ch, 1, 0, 0, Port)) {} // Очищаем буфер
  
  while (rsrecv(Ch, 1, NVT9_RECV_TO, 0, Port)) // Ждем завершающего символа предыдущего пакета
    if (Ch[0] == '\n') 
      break
  
  while (Len < Size) // Читаем
  {
    if (rsrecv(Ch, 1, NVT9_RECV_TO, 0, Port) == 0) // Читаем очередной символ
      break // Таймаут
    Buff[Len++] = Ch[0]
    if (Ch[0] == '\n') 
      return Len // Нашли хвост - выходим
  }
  return 0
}

/* Разобрать пакет */
stock bool: LibraNvt9Parse(Buff[], Len, &Weight, &WeightPow)
{
  new Value = 0
  new Pow = 0
  new bool: Neg = false
  new bool: Dot = false
  
  if (Len != NVT9_RECV_LEN) // Длина всегда 14
    return false
  
  if (Buff[0] != 'w') // Первый символ всегда 'w'
    return false  
  
  if ((Buff[1] != 'w') && (Buff[1] != 'n')) // В протоколе второй символ 'n' по логах 'w'
    return false  

    /* Значение веса */
  for (new i = 2; i <= 9; i++)
  {
    if (Buff[i] == '-') Neg = true
    if (Buff[i] == '.') Dot = true
    if ('0' <= Buff[i] <= '9')
    {
      Value *= 10
      Value += (Buff[i] - '0')
      if (Dot) Pow++
    }
  }

  if (Buff[10] != 'k')
    return false  

  if (Buff[11] != 'g')
    return false  
  
  if (Buff[12] != '\r')
    return false  
  
  if (Buff[13] != '\n')
    return false  

  if (Neg) Value = -Value
  Weight = Value
  WeightPow = Pow
  
  return true
}