/* ВЕСОПРОЦЕССОР SMART
 * Должен быть настроен F1 data format
 * (c) Copyright 2020, GLONASSsoft
 * This file is provided as is (no warranties).
 */

#pragma library SmartLibra

#include serial
#include string

#define SMART_STX 2
#define SMART_ETX 3
#define SMART_ENQ 5
#define SMART_SYN 22
#define SMART_CR 13
#define SMART_LF 10
#define SMART_ACK 6
#define SMART_NAK 21

#define US_STATUS_BIT 0x00 /* Нестабильно */
#define ST_STATUS_BIT 0x01 /* Cтабильно */
#define OL_STATUS_BIT 0x02 /* Перегрузка */
#define ER_STATUS_BIT 0x03 /* Ошибка */

#define GS_STATUS_BIT 0x00 /* Вес брутто */
#define NT_STATUS_BIT 0x04 /* Вес нетто */

/* Функция чтения периодических данных из порта формата F1*/
stock SmartReadF1(SerialPort:port, Buff[], SizeBuff)
{
    new Len, Offs = 0;
    new BuffRead[12 * 3];  
    new pBuff = 0;   
    
    Len = rsrecv(BuffRead, _, 1000, 1000, port); /* Читаем буфер */
    for (new i = 0; i < Len; i++)
    {
        if (BuffRead[i] == SMART_STX)
        {
            Offs = (i + 1);
            break;
        }
    }  
    
    while ((pBuff < SizeBuff) && (Offs < Len))
    {
        Buff[pBuff++] = BuffRead[Offs++];
        if (BuffRead[Offs] == 0x0D) break;
    }
    printf("Data len=%d", pBuff);
    return pBuff    
   
}

/* Разобрать пакет типа F1*/
stock bool:SmartParseF1(DataBuff[], DataLen, &Weight, &Pow, &StatusBits)
{
  new Value = 0;
  new bool:Pol = false
  new Bits = 0;

  if (DataLen < 11) /* Если ответ меньше 11 байт */
  {
    printf("Err.Len=%d", DataLen);
    return false;
  }
  /* Распарсиваем ответ */
  if (DataBuff[0] == '-') Pol = true;
  /* Значение веса */
  for(new i = 0; i < 7; i++)
  {
    Value = (Value * 10);
    Value += (DataBuff[1+i] - '0');
  }
  if (Pol) Value = -Value;
  switch(DataBuff[8]) /* Приводим все к кг */
  {
    case 'K':
      Pow = 0
    case 'T':
      {
        Value = Value * 1000
        Pow = 0
      }
    case 'G':
      Pow = 3
  }
  Weight = Value;
  /* Флаги с информацией о измерении */
  if (DataBuff[9] == 'N') Bits |= NT_STATUS_BIT; else Bits |= GS_STATUS_BIT;
  switch(DataBuff[10])
  {
    case ' ':
      Bits |= ST_STATUS_BIT;
    case 'M':
      Bits |= US_STATUS_BIT;
    case 'O':
      Bits |= OL_STATUS_BIT;
    case 'I':
      Bits |= ER_STATUS_BIT;
  }
  StatusBits = Bits;

  return true;
}

stock SmartSendReq(SerialPort:port)
{
  new Buff[2]

  Buff[0] = 'P'; /* Запрос массы в соответствии с выбранным форматом. На весах нужно настройить формат F1 */
  Buff[1] = SMART_CR; // <CR>
    /* Передаем запрос */
  rssend(Buff, 2, port)
}

/* Функция чтения периодических данных */
stock bool:SmartReadPeriodic(SerialPort:port, &Weight, &Pow, &StatusBits)
{
  new Buff[32]
  new Len

  /* Читаем ответ */
  Len = SmartReadF1(port, Buff, sizeof(Buff));
  /* Распарсиваем его */
  return SmartParseF1(Buff, Len, Weight, Pow, StatusBits)
}

/* Функция чтения данных с весов по запросу*/
stock bool:SmartReadRequest(SerialPort:port, &Weight, &Pow, &StatusBits, delay_rx)
{
  new Buff[32]
  new Len

  /* Передаем запрос */
  SmartSendReq(port)
  /* Задержка*/
  delay(delay_rx);
  /* Читаем ответ */
  Len = SmartReadF1(port, Buff, sizeof(Buff));
    /* Распарсиваем его */
  return SmartParseF1(Buff, Len, Weight, Pow, StatusBits)
}
