/* Весоизмерительный преобразователь ТВ-015
 * (c) Copyright 2022, GLONASSsoft
 * This file is provided as is (no warranties).
 */
 
#pragma library tv015

#include serial
#include <string>

#define CMD_GET_NET_WEIGHT    0xC2
#define CMD_GET_GROSS_WEIGHT  0xC3

#define  NET_WEIGHT           0
#define  GROSS_WEIGHT         1

#define CRC8_TABLE_SIZE       256

stock table_crc[CRC8_TABLE_SIZE]

stock crc8_generate_table(table[CRC8_TABLE_SIZE], poly)
{
  new msbit = 0x80;
  table[0] = 0;

  new t = msbit;
  for (new i = 1; i < CRC8_TABLE_SIZE; i *= 2)
  {
    t = ((t << 1) ^ (t & msbit ? poly: 0)) & 0xFF;
    for (new j = 0; j < i; ++j)
      table[i+j] = (table[j] ^ t) & 0xFF;
  }
}

stock crc8(table[CRC8_TABLE_SIZE], data[], nbytes, crc)
{
  new pdata = 0;
  while (nbytes-- > 0)
    crc = (table[(crc ^ data[pdata++]) & 0xff]) & 0xFF;

  return crc;
}

/* Формирование запроса */ 
stock Tv015GenRequest(addr, req[], reqLen, buff[64])
{
  new pBuff = 0;
  new crc = 0;
  new InterimReq[64];
  new pIReq = 0;
  
  /* Формируем запрос */
  InterimReq[pIReq++] = addr;
  for(new i = 0; i < reqLen; i++) InterimReq[pIReq++] = req[i];
  /* Считаем CRC */
  crc = crc8(table_crc, InterimReq, pIReq, crc);
  InterimReq[pIReq++] = (crc & 0xFF);
  /* Перекладываем данные в буфер*/
  buff[pBuff++] = 0xFF; // Стартовый символ
  for (new i = 0; i < pIReq; i++)
  {
    buff[pBuff++] = InterimReq[i];
    if (InterimReq[i] == 0xFF) buff[pBuff++] = 0xFE;   
  }
  buff[pBuff++] = 0xFF; // Стоп 2 раза
  buff[pBuff++] = 0xFF;
  return pBuff;
}

/* Получение ответа из принятого буфера и его проверка */
stock Tv015ParseAnswer(buff[], len, addr, Answer[], AnswerBuffSize = sizeof(Answer))
{
  new InterimAnsw[64];
  new SizeAnswer = 0;
  new i = 0;
  new crc = 0;
  
  if (len < 4) return 0; // Минимально возможный размер ответа
  if ((buff[0] != 0xFF) || (buff[1] != addr)) return 0; // Если нет стартового символа или адрес не совпал
  /* Выдергиваем данные из принятото буфера */
  i = 1
  while (i < len)
  {
    if (buff[i] != 0xFF) 
    {
      InterimAnsw[SizeAnswer++] = buff[i]
      i++;
    }
    else
    {
      if (buff[i+1] == 0xFF) break; // Если два байта FF подряд
      else if (buff[i+1] == 0xFE) InterimAnsw[SizeAnswer++] = buff[i];
      else return 0;
      i += 2;
    }
  }
  /*crc = crc8(table_crc, InterimAnsw, SizeAnswer, crc);
  printf("crc=%d", crc);*/
  if (crc8(table_crc, InterimAnsw, SizeAnswer, crc) != 0) return 0;
  /* Копируем из промежуточного буфера */
  for(i = 0; ((i < SizeAnswer) && (i < AnswerBuffSize)); i++) Answer[i] = InterimAnsw[i];
  return i;    
}

/* Получение данных */
/* Читаем вес */ 
stock bool: Tv015ReadValueWeight(addr, &Float:value, &Pow, &bool:stable, type = NET_WEIGHT)
{
  new Buff[64];
  new Len;
  new Req[1];
  new iValue;
  
  if (type == NET_WEIGHT) Req[0] = CMD_GET_NET_WEIGHT;
  else Req[0] = CMD_GET_GROSS_WEIGHT;
  Len = Tv015GenRequest(addr, Req, sizeof(Req), Buff);                  // Формируем запрос
  rssend(Buff, Len);                                                    // Передаем его
  
  Len = rsrecv(Buff, sizeof(Buff), 500, 100);                           // Читаем ответ
  if (Tv015ParseAnswer(Buff, Len, addr, Buff) == 0) return false;       // Выделяем полезные данные
  if (Buff[1] != Req[0]) return false;                                  // Если не совпал опкод запроса
  if (Buff[5] & 0x20 && Req[0] == CMD_GET_GROSS_WEIGHT) return false;   // Проверка соответствия бита
  if (!(Buff[5] & 0x20) && Req[0] == CMD_GET_NET_WEIGHT) return false;  // типа измеряемого веса
  iValue =  (((Buff[4] >> 4) & 0x0F) * 100000)
  iValue += (((Buff[4] >> 0) & 0x0F) * 10000)
  iValue += (((Buff[3] >> 4) & 0x0F) * 1000)
  iValue += (((Buff[3] >> 0) & 0x0F) * 100)
  iValue += (((Buff[2] >> 4) & 0x0F) * 10)
  iValue += (((Buff[2] >> 0) & 0x0F) * 1)
  if (Buff[5] & 0x80) iValue = -iValue;
  if (Buff[5] & 0x10) stable = true; 
  else stable = false;
  Pow = Buff[5] & 0x07;
  value = iValue;
  
  return true;
}