/* 
 * Script TV_015.p
 * ver 1.0.0
 * (c) Copyright 2022, GLONASSsoft
 * This file is provided as is (no warranties). 
 *
 * Скрипт по чтению данных весов ТВ–015
 * Параметры работы скрипта задаются через аргументы. К примеру: Addr=10
 * Addr=10 - сетевой адрес весов от 1 до 253. По умолчанию адрес равен 1; (Необходимо установить равным, установленному в весах)
 */

#include <tracker>
#include <args>
#include <time>
#include <string>
#include serial
#include TV_015

#pragma dynamic 256

#define PERIOD           1000      // период опроса в миллисекундах

#define NET_VAL_TAG      0
#define NET_REST_TAG     1
#define GROSS_VAL_TAG    2
#define GROSS_REST_TAG   3

new Addr = 0x01;                   // Сетевой адрес

@setup()
{
  crc8_generate_table(table_crc, 0x69)
  argvalue(0, "Addr", Addr);                        // Читаем сетевой адрес
  printf("Read arg. Addr=%d", Addr);
  if (Addr >= 1 && Addr <= 253) settimer(PERIOD);   // Запускаем таймер с заданным периодом
  else printf("Error!!!. Incorrect address");       // Выводим сообщение о ошибке
}

@timer()
{
  new Float:fValue = 0.0;
  new bool:Valid;
  new bool:Stable;
  new Value;
  new Pow;
  
  if (rsopen() == true)     // Удалось открыть порт
  {
    printf("Get");
    
    Valid = Tv015ReadValueWeight(Addr, fValue, Pow, Stable, NET_WEIGHT);
    Value = floatround(fValue, floatround_floor);
    settag(NET_VAL_TAG, Value, Valid, Pow);
    settag(NET_REST_TAG, Stable, Valid);
    
    Valid = Tv015ReadValueWeight(Addr, fValue, Pow, Stable, GROSS_WEIGHT);
    Value = floatround(fValue, floatround_floor);
    settag(GROSS_VAL_TAG, Value, Valid, Pow);
    settag(GROSS_REST_TAG, Stable, Valid);
    
    rsclose();
  }  
}